from dataclasses import field
from typing import TYPE_CHECKING

from rich.layout import Layout

from cyberdrop_dl.ui.progress.downloads_progress import DownloadsProgress
from cyberdrop_dl.ui.progress.file_progress import FileProgress
from cyberdrop_dl.ui.progress.scraping_progress import ScrapingProgress
from cyberdrop_dl.ui.progress.statistic_progress import DownloadStatsProgress, ScrapeStatsProgress
from cyberdrop_dl.utils.utilities import log_with_color

if TYPE_CHECKING:
    from cyberdrop_dl.managers.manager import Manager


class ProgressManager:
    def __init__(self, manager: 'Manager'):
        # File Download Bars
        self.file_progress: FileProgress = FileProgress(5, manager)

        # Scraping Printout
        self.scraping_progress: ScrapingProgress = ScrapingProgress(5, manager)

        # Overall Progress Bars & Stats
        self.download_progress: DownloadsProgress = DownloadsProgress()
        self.download_stats_progress: DownloadStatsProgress = DownloadStatsProgress()
        self.scrape_stats_progress: ScrapeStatsProgress = ScrapeStatsProgress()

        self.layout: Layout = field(init=False)

    async def startup(self) -> None:
        """Startup process for the progress manager"""
        progress_layout = Layout()
        progress_layout.split_column(
            Layout(name="upper", ratio=1, minimum_size=8),
            Layout(renderable=await self.scraping_progress.get_progress(), name="Scraping", ratio=2),
            Layout(renderable=await self.file_progress.get_progress(), name="Downloads", ratio=2),
        )
        progress_layout["upper"].split_row(
            Layout(renderable=await self.download_progress.get_progress(), name="Files", ratio=1),
            Layout(renderable=await self.scrape_stats_progress.get_progress(), name="Scrape Failures", ratio=1),
            Layout(renderable=await self.download_stats_progress.get_progress(), name="Download Failures", ratio=1),
        )

        self.layout = progress_layout

    async def print_stats(self) -> None:
        """Prints the stats of the program"""
        await log_with_color("\nDownload Stats:", "cyan", 20)
        await log_with_color(f"Downloaded {self.download_progress.completed_files} files", "green", 20)
        await log_with_color(f"Previously Downloaded {self.download_progress.previously_completed_files} files", "yellow", 20)
        await log_with_color(f"Skipped By Config {self.download_progress.skipped_files} files", "yellow", 20)
        await log_with_color(f"Failed {self.download_stats_progress.failed_files} files", "red", 20)

        scrape_failures = await self.scrape_stats_progress.return_totals()
        await log_with_color("\nScrape Failures:", "cyan", 20)
        for key, value in scrape_failures.items():
            await log_with_color(f"Scrape Failures ({key}): {value}", "red", 20)

        download_failures = await self.download_stats_progress.return_totals()
        await log_with_color("\nDownload Failures:", "cyan", 20)
        for key, value in download_failures.items():
            await log_with_color(f"Download Failures ({key}): {value}", "red", 20)


from dataclasses import field
from pathlib import Path
from typing import TYPE_CHECKING

from cyberdrop_dl.utils.utilities import sanitize_folder

if TYPE_CHECKING:
    from rich.progress import TaskID
    from yarl import URL


class MediaItem:
    def __init__(self, url: "URL", referer: "URL", download_folder: Path, filename: str, ext: str, original_filename: str):
        self.url: URL = url
        self.referer: URL = referer
        self.download_folder: Path = download_folder
        self.filename: str = filename
        self.ext: str = ext
        self.download_filename: str = field(init=False)
        self.original_filename: str = original_filename
        self.datetime: str = field(init=False)
        self.filesize: int = field(init=False)
        self.current_attempt: int = field(init=False)
        self.download_task_id: TaskID = field(init=False)


class ScrapeItem:
    def __init__(self, url: "URL", parent_title: str, part_of_album: bool = False, possible_datetime: int = None,
                 retry: bool = False, retry_path: Path = None):
        self.url: URL = url
        self.parent_title: str = parent_title
        self.part_of_album: bool = part_of_album
        self.possible_datetime: int = possible_datetime
        self.retry: bool = retry
        self.retry_path: Path = retry_path

    async def add_to_parent_title(self, title: str) -> None:
        """Adds a title to the parent title"""
        if not title or self.retry:
            return
        title = await sanitize_folder(title)
        self.parent_title = (self.parent_title + "/" + title) if self.parent_title else title



import re
from pathlib import Path

from InquirerPy import inquirer
from rich.console import Console

console = Console()


def edit_urls_prompt(URLs_File: Path, fix_strings=True) -> None:
    """Edit the URLs file"""
    console.clear()
    console.print(f"Editing URLs: {URLs_File}")
    with open(URLs_File, "r") as f:
        existing_urls = f.read()

    result = inquirer.text(
        message="URLs:", multiline=True, default=existing_urls,
        long_instruction="Press escape and then enter to finish editing.",
    ).execute()

    if fix_strings:
        result = result.replace(" ", "\n")
        result = re.sub(r"(\n)+", "\n", result)

    with open(URLs_File, "w") as f:
        f.write(result)
